@@ -12,11 +12,11 @@
 
 #define GLYR_VERSION_MAJOR "1"
 #define GLYR_VERSION_MINOR "0"
-#define GLYR_VERSION_MICRO "2"
+#define GLYR_VERSION_MICRO "6"
 
 #define GLYR_VERSION_MAJOR_INT 1
 #define GLYR_VERSION_MINOR_INT 0
-#define GLYR_VERSION_MICRO_INT 2
+#define GLYR_VERSION_MICRO_INT 6
 
 /**
  * GLYR_CHECK_VERSION:
@@ -37,7 +37,7 @@
 #define GLYR_CHECK_VERSION(X,Y,Z) (X <= GLYR_VERSION_MAJOR_INT || Y <= GLYR_VERSION_MINOR_INT || Z <= GLYR_VERSION_MICRO_INT)
 
 #define GLYR_VERSION_NAME  "Raving Raven"
-#define GLYR_DEBUG         TRUE
+#define GLYR_DEBUG         FALSE
 
 /* Message output */
 #define GLYR_OUTPUT stderr
