@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #ifdef __linux__
 /* Backtrace*/
@@ -78,12 +79,12 @@ typedef struct
 //////////////////////////
 
 #define INIT_CALLBACK_DATA \
-    {                          \
-        .output_path  = NULL,  \
-                        .exec_on_call = NULL,  \
-                                        .as_one       = false, \
-                                                .append_format= false  \
-    }                          \
+{                          \
+    .output_path  = NULL,  \
+    .exec_on_call = NULL,  \
+    .as_one       = false, \
+    .append_format= false  \
+}                          \
                                                      
 //////////////////////////
 
@@ -173,6 +174,43 @@ static void handle_cancel (int signo)
 
 ////////////////////////////////////////////////////////////////
 
+static char * create_cache_directory (void)
+{
+    char * xdg_cache_dir = g_strdup(g_getenv("XDG_CACHE_HOME"));
+    if(xdg_cache_dir == NULL) {
+        xdg_cache_dir = (char *)g_getenv("HOME");
+        if(xdg_cache_dir == NULL) {
+            xdg_cache_dir = g_strdup("/tmp");
+        } else {
+            xdg_cache_dir = g_strdup_printf("%s%c.cache",
+                xdg_cache_dir,
+                G_DIR_SEPARATOR
+            );
+        }
+    }
+
+    char * full_path = g_strdup_printf("%s%cglyrc",
+        xdg_cache_dir,
+        G_DIR_SEPARATOR
+    );
+
+    if(!g_file_test(full_path, G_FILE_TEST_IS_DIR)) {
+        g_printerr("<%s> does not exist yet. Creating...", full_path);
+        if(g_mkdir_with_parents(full_path, 0777)  != 0) {
+            g_printerr("Could not mkdir cache directory at %s\n", full_path);
+            exit(EXIT_FAILURE);
+        } else {
+            g_printerr("done.\n");
+        }
+        
+    }
+
+    g_free(xdg_cache_dir);
+    return full_path;
+}
+
+////////////////////////////////////////////////////////////////
+
 static void init_cancel_handler (GlyrQuery * p_lastQuery)
 {
     glob_lastQuery = p_lastQuery;
@@ -181,7 +219,7 @@ static void init_cancel_handler (GlyrQuery * p_lastQuery)
 
 ////////////////////////////////////////////////////////////////
 // --------------------------------------------------------- //
-// --------------------------------------------------------- //
+///////////////////////////////////////////////////////////////
 
 static void print_version (GlyrQuery * s)
 {
@@ -283,6 +321,8 @@ void help_short (GlyrQuery * s)
             IN"    glyrc cache list --cache /tmp # List all in the cache\n"
             IN"    glyrc cache delete cover -a Equilibrium -b Sagas --cache /tmp # Delete artist/album\n"
             IN"    glyrc cache select lyrics -a Knorkator -t 'A' -n 2 --cache /tmp # Search for two items in cache and print them\n"
+            IN"\n"
+            IN"The return code will be 0 on success, 1 on any failure. No distinction is made which error happened.\n"
             IN"\n\n"
             "With each item received you get a link to the original source, please refer to the individual terms of use,\n"
             "copying and distributing of this data might be not allowed.\n"
@@ -358,7 +398,7 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
         {"verbosity",     required_argument, 0, 'v'},
         {"qsratio",       required_argument, 0, 'q'},
         {"formats",       required_argument, 0, 'F'},
-        {"cache",         required_argument, 0, 'c'},
+        {"cache",         optional_argument, 0, 'c'},
         {"help",          no_argument,       0, 'h'},
         {"version",       no_argument,       0, 'V'},
         {"download",      no_argument,       0, 'd'},
@@ -389,7 +429,7 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
     {
         gint c;
         gint option_index = 0;
-        if ( (c = getopt_long (argc, argv, "N:f:W:w:p:r:m:x:u:v:q:c:F:hVodDLa:b:t:i:e:s:n:l:z:j:k:8gGyY",long_options, &option_index) ) == -1)
+        if ( (c = getopt_long (argc, argv, "N:f:W:w:p:r:m:x:u:v:q:c::F:hVodDLa:b:t:i:e:s:n:l:z:j:k:8gGyY",long_options, &option_index) ) == -1)
         {
             break;
         }
@@ -401,9 +441,9 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
             char * dirname = g_path_get_dirname (optarg);
             gsize opt_len = strlen (optarg);
             if (g_ascii_strncasecmp (optarg,"stdout",opt_len) == 0 ||
-                    g_ascii_strncasecmp (optarg,"stderr",opt_len) == 0 ||
-                    g_ascii_strncasecmp (optarg,"null",  opt_len) == 0 ||
-                    g_file_test (dirname,G_FILE_TEST_IS_DIR | G_FILE_TEST_EXISTS) == TRUE)
+                g_ascii_strncasecmp (optarg,"stderr",opt_len) == 0 ||
+                g_ascii_strncasecmp (optarg,"null",  opt_len) == 0 ||
+                g_file_test (dirname,G_FILE_TEST_IS_DIR | G_FILE_TEST_EXISTS) == TRUE)
             {
                 CBData->output_path = optarg;
                 g_free (dirname);
@@ -472,7 +512,12 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
             break;
         case 'c':
         {
-            GlyrDatabase * new_db = glyr_db_init (optarg);
+            char * path = g_strdup(optarg);
+            if(path == NULL) {
+                path = create_cache_directory();
+            }
+
+            GlyrDatabase * new_db = glyr_db_init (path);
             if (db == NULL)
             {
                 cprint (DEFAULT,-1,NULL,"Unable to open or create a database at specified path.\n");
@@ -481,6 +526,7 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
             *db = new_db;
             glyr_opt_db_autowrite (glyrs,TRUE);
             glyr_opt_lookup_db (glyrs,new_db);
+            g_free(path);
         }
         break;
         case 'l':
@@ -552,16 +598,6 @@ static void parse_commandline_general (int argc, char * const * argv, GlyrQuery
             exit (-1);
         } // end switch()
     } // end while()
-
-    if (optind < argc && glyrs->verbosity >= 2)
-    {
-        cprint (DEFAULT,2,glyrs,"Note: You have unused arguments:\n");
-        for (int i = optind; i < argc; ++i)
-        {
-            cprint (DEFAULT,2,glyrs,"      * %s\n",argv[optind]);
-        }
-        cprint (DEFAULT,2,glyrs,"\n\n");
-    }
 }
 
 ////////////////////////////////////////////////////////////////
@@ -582,8 +618,8 @@ static GLYR_ERROR callback (GlyrMemCache * c, GlyrQuery * s)
     {
         gsize write_len = strlen (CBQueryData->output_path);
         if (g_ascii_strncasecmp (CBQueryData->output_path,"stdout",write_len) == 0 ||
-                g_ascii_strncasecmp (CBQueryData->output_path,"stderr",write_len) == 0 ||
-                g_ascii_strncasecmp (CBQueryData->output_path,"null",  write_len) == 0)
+            g_ascii_strncasecmp (CBQueryData->output_path,"stderr",write_len) == 0 ||
+            g_ascii_strncasecmp (CBQueryData->output_path,"null",  write_len) == 0)
         {
             glyr_cache_write (c,CBQueryData->output_path);
         }
@@ -704,7 +740,7 @@ static void do_iterate_over_db (GlyrDatabase * db)
 
 //////////////////////////////////////
 
-static void do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatabase * db)
+static int do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatabase * db, callback_data_t * cbdata)
 {
     const char list_op[] = "list";
     const char slct_op[] = "select";
@@ -715,7 +751,7 @@ static void do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatab
     if (db == NULL)
     {
         cvprint (RED,"No database specified. Use --cache.\n");
-        return;
+        return EXIT_FAILURE;
     }
 
     ////////////////////////////////
@@ -723,8 +759,7 @@ static void do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatab
     if (g_ascii_strncasecmp (operation,list_op,sizeof (list_op) - 1) == 0)
     {
         do_iterate_over_db (db);
-
-        return;
+        return EXIT_SUCCESS;
     }
 
     ////////////////////////////////
@@ -733,8 +768,7 @@ static void do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatab
     {
         int killed_items = glyr_db_delete (db,q);
         cvprint (BLUE,"Deleted %d items from the db.",killed_items);
-
-        return;
+        return killed_items > 0;
     }
 
     ////////////////////////////////
@@ -744,15 +778,20 @@ static void do_cache_interface (const char * operation, GlyrQuery * q, GlyrDatab
         GlyrMemCache * list = glyr_db_lookup (db,q);
         GlyrMemCache * iter = list;
 
-        while (iter != NULL)
-        {
-            glyr_cache_print (iter);
-            cvprint (BLUE,"//////////////////////\n");
+        if(iter == NULL)  {
+            return EXIT_FAILURE;
+        }
+
+        q->callback.user_pointer = cbdata;
+        while (iter != NULL) {
+            callback(iter, q);
             iter = iter->next;
         }
 
-        return;
+        return EXIT_SUCCESS;
     }
+
+    return EXIT_FAILURE;
 }
 
 
@@ -846,8 +885,8 @@ int main (int argc, char * argv[])
     atexit (global_cleanup);
     init_signals();
 
-    /* Assume success */
-    gint exit_code = EXIT_SUCCESS;
+    /* Assume failure, as always in life */
+    gint exit_code = EXIT_FAILURE;
 
     /* Init / Destroy of libglyr.
      * You _have_ to call this before making any calls
@@ -885,6 +924,8 @@ int main (int argc, char * argv[])
 
             if (my_list != NULL)
             {
+                exit_code = EXIT_SUCCESS;
+
                 if (CBQueryData.as_one && glyr_type_is_image (my_query.type) == false)
                 {
                     GlyrMemCache * concat = concatenate_list (my_list);
@@ -922,16 +963,18 @@ int main (int argc, char * argv[])
             GlyrQuery db_query;
             GlyrDatabase * db = NULL;
             callback_data_t CBQueryData = INIT_CALLBACK_DATA;
+            SET_IF_NULL (CBQueryData.output_path,".");
 
             glyr_query_init (&db_query);
+            glyr_opt_verbosity (&db_query,2);
 
-            if (argc >= 4)
+            if (argc >= 4 && strcmp("list", argv[2]) != 0)
             {
                 parse_type_argument (argv[3],&db_query);
             }
 
             parse_commandline_general (argc-2, argv+2,&db_query,&CBQueryData,&db);
-            do_cache_interface (argv[2],&db_query,db);
+            do_cache_interface (argv[2],&db_query,db, &CBQueryData);
         }
         else
         {
